/**
 * @file Serial.cpp
 * @brief Provides Arduino Serial funtions on the Propeller
 *
 * Copyright (c) 2013 by Martin Heermance
 * MIT Licensed
 */


#include "Arduino.h"
#include <stdio.h>

void Serial_::setdevice(fdserial *dev)
{
    if(dev) debug = dev;
    simpleterm_set(debug);
}

int Serial_::available(void)
{
    return fdserial_rxReady(debug);
}

void Serial_::begin(unsigned long baud)
{
    if(debug) fdserial_close(debug);
    simpleterm_set(fdserial_open(31,30,0,baud));
    debug = simpleterm_pointer();
}

void Serial_::end(void)
{
    fdserial_close(debug);
}

void Serial_::flush(void)
{
    fdserial_rxFlush(debug);
}

int Serial_::peek(void)
{
    return fdserial_rxPeek(debug);
}
  
int Serial_::read(void)
{
    return readChar(debug);
}

size_t Serial_::write(uint8_t ch)
{
  writeChar(debug,ch);
  return 1;
}

size_t Serial_::write(unsigned long n)
{
  writeDec(debug,(int)n);
  return 1;
}

size_t Serial_::write(long n)
{
  writeDec(debug,(int)n);
  return 1;
}
size_t Serial_::write(unsigned int n)
{ 
  writeDec(debug,(int)n);
  return 1;
}

size_t Serial_::write(int n)
{
  writeDec(debug,(int)n);
  return 1;
}
  
Serial_::operator bool() {
    delay(1000);
    return true;
}
      
Serial_ Serial;

/*
void Serial_::print(const char * str)
{
    putStr(str);
}

void Serial_::print(const char ch)
{
    putChar(ch);
}

void Serial_::print(int num, int type)
{
    switch(type) {
        case BIN:
            putBin(num);
            break;
        case DEC:
            putDec(num);
            break;
        case HEX:
            putHex(num);
            break;
        case OCT:
            printNumber(simpleterm_pointer(), num, 8, 0, '0');
            break;
    }    
}

void Serial_::println(long num)
{
    putDec(num);
}

void Serial_::println(const char * str)
{
    putStr(str);
}
*/
